@@ -1,9 +1,8 @@
 /*  This file is part of "xtrace"
  *  Copyright (C) 2005,2006 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,11 +15,11 @@
  */
 #include <config.h>
 
-// to get strndup
-#define _GNU_SOURCE
 #include <assert.h>
 #include <values.h>
+#include <stdint.h>
 #include <sys/types.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -65,7 +64,7 @@ struct constant {
 typedef bool request_func(struct connection*,bool,bool,struct expectedreply *);
 typedef void reply_func(struct connection*,bool*,bool*,void*);
 
-struct request { 
+struct request {
 	const char *name;
 	const struct parameter *parameters;
 	const struct parameter *answers;
@@ -107,7 +106,7 @@ void free_usedextensions(struct usedextension *e) {
 
 struct expectedreply {
 	struct expectedreply *next;
-	u_int64_t seq;
+	uint64_t seq;
 	const struct request *from;
 	void *data;
 };
@@ -121,21 +120,21 @@ static void print_bitfield(const char *name,const struct constant *constants, un
 
 	/* bitmasks should have some */
 	assert(constants != NULL);
-	printf("%s=",name);
+	fprintf(out,"%s=",name);
 
 	for( c = constants; c->name != NULL ; c++ ) {
 		if( c->value == 0 )
 			zeroname = c->name;
 		else if( (l & c->value) != 0 ) {
 			if( !first )
-				putchar(',');
+				putc(',',out);
 			first = false;
-			fputs(c->name,stdout);
+			fputs(c->name,out);
 		}
 		
 	}
 	if( first )
-		fputs(zeroname,stdout);
+		fputs(zeroname,out);
 };
 
 static const char *findConstant(const struct constant *constants, unsigned long l){
@@ -161,7 +160,7 @@ struct parameter {
 	 * in this parameter-list. */
 	size_t offse;
 	const char *name;
-	enum fieldtype { 
+	enum fieldtype {
 		/* signed endian specific: */
 		ft_INT8, ft_INT16, ft_INT32,
 		/* unsigned decimal endian specific: */
@@ -180,13 +179,13 @@ struct parameter {
 		/* Different forms of lists: */
 		/*	- boring ones */
 		ft_STRING8, ft_LISTofCARD32, ft_LISTofATOM,
-		ft_LISTofCARD8, ft_LISTofCARD16, 
-		ft_LISTofUINT8, ft_LISTofUINT16, 
-		ft_LISTofUINT32, 
+		ft_LISTofCARD8, ft_LISTofCARD16,
+		ft_LISTofUINT8, ft_LISTofUINT16,
+		ft_LISTofUINT32,
 		/*	- one of the above depening on last FORMAT */
 		ft_LISTofFormat,
 		/*	- iterate of list description in constants field */
-		ft_LISTofStruct, 
+		ft_LISTofStruct,
 		/*	- same but length is mininum length and
 		 *	  actual length is taken from end of last list
 		 *	  or LASTMARKER */
@@ -196,16 +195,16 @@ struct parameter {
 		 *	  offset */
 		ft_LISTofVALUE,
 		/* an LISTofStruct with count = 1 */
-		ft_Struct, 
+		ft_Struct,
 		/* specify bits per item for LISTofFormat */
 		ft_FORMAT8,
-		/* an event 
+		/* an event
 		 * (would have also been possible with Struct and many IF)*/
-		ft_EVENT, 
+		ft_EVENT,
 		/* jump to other parameter list if matches */
-		ft_IF8, 
+		ft_IF8,
 		/* jump to other parameter list if matches atom name */
-		ft_IFATOM, 
+		ft_IFATOM,
 		/* set end of last list manually, (for LISTofVarStruct) */
 		ft_LASTMARKER,
 		/* a ft_CARD32 looking into the ATOM list */
@@ -227,26 +226,26 @@ static size_t printSTRING8(u8 *buffer,size_t buflen,const struct parameter *p,si
 		len = buflen - ofs;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s='",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s='",p->name);
 	while( len > 0 ) {
 		if( nr == maxshownlistlen ) {
-			fputs("'...",stdout);
+			fputs("'...",out);
 		} else if( nr < maxshownlistlen ) {
 			unsigned char c = getCARD8(ofs);
 			if( c == '\n' ) {
-				putchar('\\');putchar('n');
+				putc('\\',out);putc('n',out);
 			} else if( c == '\t' ) {
-				putchar('\\');putchar('t');
+				putc('\\',out);putc('t',out);
 			} else if( (c >= ' ' && c <= '~' ) )
-				putchar(c);
+				putc(c,out);
 			else
-				printf("\\%03hho", c);
+				fprintf(out,"\\%03hho", c);
 		}
 		ofs++;len--;nr++;
 	}
 	if( nr <= maxshownlistlen )
-		putchar('\'');
+		putc('\'',out);
 	return ofs;
 }
 
@@ -260,28 +259,28 @@ static size_t printLISTofCARD8(u8 *buffer,size_t buflen,const struct parameter *
 		len = buflen - ofs;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
 		unsigned char u8;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u8 = getCARD8(ofs);
 			value = findConstant(p->constants,u8);
 			if( value )
-				printf("%s(0x%hhx)",value,u8);
+				fprintf(out,"%s(0x%hhx)",value,u8);
 			else
-				printf("0x%02hhx",u8);
+				fprintf(out,"0x%02hhx",u8);
 		}
 		len--;ofs++;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -295,28 +294,28 @@ static size_t printLISTofCARD16(struct connection *c,u8 *buffer,size_t buflen,co
 		len = (buflen - ofs)/2;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
-		u_int16_t u16;
+		uint16_t u16;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u16 = getCARD16(ofs);
 			value = findConstant(p->constants,u16);
 			if( value )
-				printf("%s(0x%hx)",value,(unsigned short int)u16);
+				fprintf(out,"%s(0x%hx)",value,(unsigned short int)u16);
 			else
-				printf("0x%04hx",(unsigned short int)u16);
+				fprintf(out,"0x%04hx",(unsigned short int)u16);
 		}
 		len--;ofs+=2;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -330,28 +329,28 @@ static size_t printLISTofCARD32(struct connection *c,u8 *buffer,size_t buflen,co
 		len = (buflen - ofs)/4;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
-		u_int32_t u32;
+		uint32_t u32;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u32 = getCARD32(ofs);
 			value = findConstant(p->constants,u32);
 			if( value )
-				printf("%s(0x%x)",value,(unsigned int)u32);
+				fprintf(out,"%s(0x%x)",value,(unsigned int)u32);
 			else
-				printf("0x%08x",(unsigned int)u32);
+				fprintf(out,"0x%08x",(unsigned int)u32);
 		}
 		len--;ofs+=4;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -365,30 +364,30 @@ static size_t printLISTofATOM(struct connection *c,u8 *buffer,size_t buflen,cons
 		len = (buflen - ofs)/4;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
-		u_int32_t u32;
+		uint32_t u32;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u32 = getCARD32(ofs);
 			value = findConstant(p->constants,u32);
 			if( value )
-				printf("%s(0x%x)",value,(unsigned int)u32);
+				fprintf(out,"%s(0x%x)",value,(unsigned int)u32);
 			else if( (value = getAtom(c,u32)) == NULL )
-				printf("0x%x",(unsigned int)u32);
+				fprintf(out,"0x%x",(unsigned int)u32);
 			else
-				printf("0x%x(\"%s\")",(unsigned int)u32,value);
+				fprintf(out,"0x%x(\"%s\")",(unsigned int)u32,value);
 		}
 		len--;ofs+=4;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -402,28 +401,28 @@ static size_t printLISTofUINT8(u8 *buffer,size_t buflen,const struct parameter *
 		len = buflen - ofs;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
 		unsigned char u8;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u8 = getCARD8(ofs);
 			value = findConstant(p->constants,u8);
 			if( value )
-				printf("%s(%d)",value,(unsigned int)u8);
+				fprintf(out,"%s(%d)",value,(unsigned int)u8);
 			else
-				printf("%d",(unsigned int)u8);
+				fprintf(out,"%d",(unsigned int)u8);
 		}
 		len--;ofs++;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -437,28 +436,28 @@ static size_t printLISTofUINT16(struct connection *c,u8 *buffer,size_t buflen,co
 		len = (buflen - ofs)/2;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
-		u_int16_t u16;
+		uint16_t u16;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u16 = getCARD16(ofs);
 			value = findConstant(p->constants,u16);
 			if( value )
-				printf("%s(%d)",value,(unsigned int)u16);
+				fprintf(out,"%s(%d)",value,(unsigned int)u16);
 			else
-				printf("%d",(unsigned int)u16);
+				fprintf(out,"%d",(unsigned int)u16);
 		}
 		len--;ofs+=2;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -472,28 +471,28 @@ static size_t printLISTofUINT32(struct connection *c,u8 *buffer,size_t buflen,co
 		len = (buflen - ofs)/4;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( len > 0 ) {
 		const char *value;
-		u_int32_t u32;
+		uint32_t u32;
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
 			u32 = getCARD32(ofs);
 			value = findConstant(p->constants,u32);
 			if( value )
-				printf("%s(%x)",value,(unsigned int)u32);
+				fprintf(out,"%s(%x)",value,(unsigned int)u32);
 			else
-				printf("%x",(unsigned int)u32);
+				fprintf(out,"%x",(unsigned int)u32);
 		}
 		len--;ofs+=4;nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -517,10 +516,10 @@ static size_t printLISTofVALUE(struct connection *c,u8 *buffer,size_t buflen,con
 	if( ofs > buflen )
 		return ofs;
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s={",param->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s={",param->name);
 	while( buflen > ofs && buflen-ofs >= 4 ) {
-		u_int32_t u32; u_int16_t u16; u_int8_t u8;
+		uint32_t u32; uint16_t u16; uint8_t u8;
 		int32_t i32; int16_t i16; int8_t i8;
 		const char *constant;
 
@@ -531,7 +530,7 @@ static size_t printLISTofVALUE(struct connection *c,u8 *buffer,size_t buflen,con
 			continue;
 		}
 		if( notfirst )
-			putchar(' ');
+			putc(' ',out);
 		notfirst = true;
 		/* this is funny, but that is the protocol... */
 		u32 = getCARD32(ofs); i32 = u32;
@@ -555,63 +554,63 @@ static size_t printLISTofVALUE(struct connection *c,u8 *buffer,size_t buflen,con
 			constant = findConstant(v->constants,u32);
 			break;
 		}
-		fputs(v->name,stdout);putchar('=');
+		fputs(v->name,out);putc('=',out);
 		if( constant != NULL ) {
-			fputs(constant,stdout);
-			putchar('(');
+			fputs(constant,out);
+			putc('(',out);
 		}
 		switch( v->type ) {
 		 case ft_INT8:
-			 printf("%d",(int)i8);
+			 fprintf(out,"%d",(int)i8);
 			 break;
 		 case ft_INT16:
-			 printf("%d",(int)i16);
+			 fprintf(out,"%d",(int)i16);
 			 break;
 		 case ft_INT32:
-			 printf("%d",(int)i32);
+			 fprintf(out,"%d",(int)i32);
 			 break;
 		 case ft_UINT8:
-			 printf("%u",(unsigned int)u8);
+			 fprintf(out,"%u",(unsigned int)u8);
 			 break;
 		 case ft_UINT16:
-			 printf("%u",(unsigned int)u16);
+			 fprintf(out,"%u",(unsigned int)u16);
 			 break;
 		 case ft_UINT32:
-			 printf("%u",(unsigned int)u32);
+			 fprintf(out,"%u",(unsigned int)u32);
 			 break;
 		 case ft_ENUM8:
 			 if( constant == NULL )
-				 fputs("unknown:",stdout);
+				 fputs("unknown:",out);
 		 case ft_CARD8:
-			 printf("0x%02x",(unsigned int)u8);
+			 fprintf(out,"0x%02x",(unsigned int)u8);
 			 break;
 		 case ft_ENUM16:
 			 if( constant == NULL )
-				 fputs("unknown:",stdout);
+				 fputs("unknown:",out);
 		 case ft_CARD16:
-			 printf("0x%04x",(unsigned int)u16);
+			 fprintf(out,"0x%04x",(unsigned int)u16);
 			 break;
 		 case ft_ATOM:
-			 printf("0x%x",(unsigned int)u32);
+			 fprintf(out,"0x%x",(unsigned int)u32);
 			 atom = getAtom(c, u32);
 			 if( atom != NULL )
-				 printf("(\"%s\")", atom);
+				 fprintf(out,"(\"%s\")", atom);
 			 break;
 		 case ft_ENUM32:
 			 if( constant == NULL )
-				 fputs("unknown:",stdout);
+				 fputs("unknown:",out);
 		 case ft_CARD32:
-			 printf("0x%08x",(unsigned int)u32);
+			 fprintf(out,"0x%08x",(unsigned int)u32);
 			 break;
 		 default:
 			 assert(0);
 		}
 		if( constant != NULL ) {
-			putchar(')');
+			putc(')',out);
 		}
 		ofs += 4; v++;
 	}
-	putchar('}');
+	putc('}',out);
 	/* TODO: print error if flags left or v!=EOV? */
 	return ofs;
 }
@@ -623,7 +622,7 @@ struct stack {
 };
 
 static unsigned long getFromStack(struct stack *stack, size_t depth) {
-	assert(stack != NULL && stack->ofs > depth );
+	assert(stack != NULL && stack->ofs > (int)depth );
 	return stack->base[stack->ofs - 1 - depth];
 }
 
@@ -651,28 +650,28 @@ static size_t printLISTofStruct(struct connection *c,u8 *buffer,size_t buflen,co
 	substruct++;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( buflen > ofs && buflen-ofs >= len && count > 0) {
 
 		if( nr == maxshownlistlen ) {
-			fputs(",...",stdout);
+			fputs(",...",out);
 			if( len == 0 )
 				ofs = SIZE_MAX;
 			break;
 		} else if( nr < maxshownlistlen ) {
 			if( notfirst )
-				putchar(',');
+				putc(',',out);
 			notfirst = true;
-			putchar('{');
+			putc('{',out);
 
 			print_parameters(c,buffer+ofs,len,substruct,false,stack);
 
-			putchar('}');
+			putc('}',out);
 		}
 		ofs += len; count--; nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 static size_t printLISTofVarStruct(struct connection *c,u8 *buffer,size_t buflen,const struct parameter *p,size_t count, size_t ofs, struct stack *stack){
@@ -688,28 +687,28 @@ static size_t printLISTofVarStruct(struct connection *c,u8 *buffer,size_t buflen
 	substruct++;
 
 	if( print_offsets )
-		printf("[%d]",(int)ofs);
-	printf("%s=",p->name);
+		fprintf(out,"[%d]",(int)ofs);
+	fprintf(out,"%s=",p->name);
 	while( buflen > ofs && buflen-ofs >= len && count > 0) {
 		size_t lentoadd;
 
 		if( nr >= maxshownlistlen ) {
-			fputs(",...;",stdout);
+			fputs(",...;",out);
 			/* there is nothing here to calculate the rest,
 			 * so just return the unreachable */
 			return SIZE_MAX;
 		}
 		if( notfirst )
-			putchar(',');
+			putc(',',out);
 		notfirst = true;
-		putchar('{');
+		putc('{',out);
 
 		lentoadd = print_parameters(c,buffer+ofs,buflen-ofs,substruct,false,stack);
 
-		putchar('}');
+		putc('}',out);
 		ofs += lentoadd; count--; nr++;
 	}
-	putchar(';');
+	putc(';',out);
 	return ofs;
 }
 
@@ -727,10 +726,10 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 	for( p = parameters; p->name != NULL; p++ ) {
 		int8_t i8; int16_t i16; int32_t i32;
 #ifdef STUPIDCC
-		u_int8_t u8=0; u_int16_t u16=0; u_int32_t u32=0;
+		uint8_t u8=0; uint16_t u16=0; uint32_t u32=0;
 		unsigned long l=0;
 #else
-		u_int8_t u8; u_int16_t u16; u_int32_t u32;
+		uint8_t u8; uint16_t u16; uint32_t u32;
 		unsigned long l;
 #endif
 		size_t ofs;
@@ -746,11 +745,11 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 			ofs = p->offse;
 
 		if( notfirst )
-			putchar(' ');
+			putc(' ',out);
 		notfirst = true;
 
 		if( p->type == ft_IF8 ) {
-			if( ofs < len && 
+			if( ofs < len &&
 			  /* some more overloading: */
 			  getCARD8(ofs) == (unsigned char)(p->name[0]) )
 				p = ((struct parameter *)p->constants)-1;
@@ -838,23 +837,23 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 			if( ofs + 4 > len )
 				continue;
 			if( print_offsets )
-				printf("[%d]",(int)ofs);
-			fputs(p->name,stdout);putchar('=');
+				fprintf(out,"[%d]",(int)ofs);
+			fputs(p->name,out);putc('=',out);
 			u32 = getCARD32(ofs);
 			value = findConstant(p->constants,u32);
 			atom = getAtom(c, u32);
 			if( value != NULL )
-				printf("%s(0x%x)",value, (unsigned int)u32);
+				fprintf(out,"%s(0x%x)",value, (unsigned int)u32);
 			else if( atom == NULL )
-				printf("0x%x(unrecognized atom)",(unsigned int)u32);
+				fprintf(out,"0x%x(unrecognized atom)",(unsigned int)u32);
 			else
-				printf("0x%x(\"%s\")",(unsigned int)u32, atom);
+				fprintf(out,"0x%x(\"%s\")",(unsigned int)u32, atom);
 			continue;
 		 case ft_BE32:
 			if( ofs + 4 > len )
 				continue;
-			fputs(p->name,stdout);putchar('=');
-			printf("0x%08x",(unsigned int)getBE32(ofs));
+			fputs(p->name,out);putc('=',out);
+			fprintf(out,"0x%08x",(unsigned int)getBE32(ofs));
 			continue;
 		 case ft_GET:
 			stored = getFromStack(&newstack,p->offse);
@@ -863,7 +862,7 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 			break;
 		}
 		assert( p->type <= ft_BITMASK32);
-		 
+		
 		if( ((ofs+4)&~3) > len )
 			/* this field is missing */
 			continue;
@@ -899,57 +898,57 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 		}
 		value = findConstant(p->constants,l);
 		if( print_offsets )
-			printf("[%d]",(int)ofs);
-		fputs(p->name,stdout);putchar('=');
+			fprintf(out,"[%d]",(int)ofs);
+		fputs(p->name,out);putc('=',out);
 		if( value != NULL ) {
-			fputs(value,stdout);
-			putchar('(');
+			fputs(value,out);
+			putc('(',out);
 		}
 		switch( p->type ) {
 		 case ft_INT8:
 			 i8 = u8;
-			 printf("%d",(int)i8);
+			 fprintf(out,"%d",(int)i8);
 			 break;
 		 case ft_INT16:
 			 i16 = u16;
-			 printf("%d",(int)i16);
+			 fprintf(out,"%d",(int)i16);
 			 break;
 		 case ft_INT32:
 			 i32 = u32;
-			 printf("%d",(int)i32);
+			 fprintf(out,"%d",(int)i32);
 			 break;
 		 case ft_PUSH8:
 		 case ft_STORE8:
 		 case ft_UINT8:
-			 printf("%u",(unsigned int)u8);
+			 fprintf(out,"%u",(unsigned int)u8);
 			 break;
 		 case ft_PUSH16:
 		 case ft_STORE16:
 		 case ft_UINT16:
-			 printf("%u",(unsigned int)u16);
+			 fprintf(out,"%u",(unsigned int)u16);
 			 break;
 		 case ft_PUSH32:
 		 case ft_STORE32:
 		 case ft_UINT32:
-			 printf("%u",(unsigned int)u32);
+			 fprintf(out,"%u",(unsigned int)u32);
 			 break;
 		 case ft_ENUM8:
 			 if( value == NULL )
-				 fputs("unknown:",stdout);
+				 fputs("unknown:",out);
 		 case ft_CARD8:
-			 printf("0x%02x",(unsigned int)u8);
+			 fprintf(out,"0x%02x",(unsigned int)u8);
 			 break;
 		 case ft_ENUM16:
 			 if( value == NULL )
-				 fputs("unknown:",stdout);
+				 fputs("unknown:",out);
 		 case ft_CARD16:
-			 printf("0x%04x",(unsigned int)u16);
+			 fprintf(out,"0x%04x",(unsigned int)u16);
 			 break;
 		 case ft_ENUM32:
 			 if( value == NULL )
-				 fputs("unknown:",stdout);
+				 fputs("unknown:",out);
 		 case ft_CARD32:
-			 printf("0x%08x",(unsigned int)u32);
+			 fprintf(out,"0x%08x",(unsigned int)u32);
 			 break;
 		 case ft_BITMASK8:
 		 case ft_BITMASK16:
@@ -978,7 +977,7 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 			 assert(0);
 		}
 		if( value != NULL ) {
-			putchar(')');
+			putc(')',out);
 		}
 	}
 	pop(&newstack,oldstack);
@@ -988,7 +987,7 @@ static size_t print_parameters(struct connection *c,const unsigned char *buffer,
 /* replace ra(GrabButton) in requests.inc by ra2(GrabButton)
  * and add this function and all
  * GrabButton requests will have an AnyModifier set before
- * being forwarded to the server... >:-] 
+ * being forwarded to the server... >:-]
  *
 static bool requestGrabButton(struct connection *c, bool pre, bool bigrequest,struct expectedreply *reply) {
 	if( !pre )
@@ -1014,7 +1013,7 @@ static bool requestQueryExtension(struct connection *c, bool pre, bool bigreques
 }
 
 static bool requestInternAtom(struct connection *c, bool pre, bool bigrequest UNUSED, struct expectedreply *reply) {
-	u_int16_t len;
+	uint16_t len;
 	if( pre )
 		return false;
 	if( reply == NULL)
@@ -1034,7 +1033,7 @@ static void replyListFontsWithInfo(struct connection *c,bool *ignore,bool *dontr
 	unsigned int seq = serverCARD16(2);
 	if( serverCARD8(1) == 0 ) {
 
-		printf("%03d:>:0x%04x:%u: Reply to ListFontsWithInfo: end of list\n", c->id, seq, c->serverignore);
+		fprintf(out,"%03d:>:0x%04x:%u: Reply to ListFontsWithInfo: end of list\n", c->id, seq, c->serverignore);
 		*ignore = true;
 	} else
 		*dontremove = true;
@@ -1059,7 +1058,7 @@ static void replyQueryExtension(struct connection *c,bool *ignore UNUSED,bool *d
 }
 
 static void replyInternAtom(struct connection *c,bool *ignore UNUSED,bool *dontremove UNUSED,void *data) {
-	u_int32_t atom;
+	uint32_t atom;
 	if( data == NULL )
 		return;
 	atom = serverCARD32(8);
@@ -1125,7 +1124,7 @@ static inline void print_client_request(struct connection *c,bool bigrequest) {
 	else
 		ignore = r->request_func(c,true,bigrequest,NULL);
 	if( !ignore ) {
-		printf("%03d:<:%04x:%3u: %sRequest(%hhu): %s ",
+		fprintf(out,"%03d:<:%04x:%3u: %sRequest(%hhu): %s ",
 				c->id,(unsigned int)(c->seq),c->clientignore,
 				extensionname,req,
 				r->name
@@ -1133,10 +1132,10 @@ static inline void print_client_request(struct connection *c,bool bigrequest) {
 		if( r->parameters != NULL )
 			print_parameters(c,c->clientbuffer,len,r->parameters, bigrequest, &stack);
 		else
-			puts("obsolete without parameter description");
+			fputs("obsolete without parameter description\n",out);
 		if( r->request_func != NULL )
 			(void)r->request_func(c,false,bigrequest,NULL);
-		putchar('\n');
+		putc('\n',out);
 	}
 	if( r->answers != NULL ) {
 		/* register an awaited response */
@@ -1154,9 +1153,9 @@ static inline void print_client_request(struct connection *c,bool bigrequest) {
 
 static inline void print_server_event(struct connection *c) {
 
-	printf("%03d:>:%04llx: Event ",c->id,(unsigned long long)c->seq);
+	fprintf(out,"%03d:>:%04llx: Event ",c->id,(unsigned long long)c->seq);
 	print_event(c,c->serverbuffer);
-	putchar('\n');
+	putc('\n',out);
 }
 
 
@@ -1176,7 +1175,7 @@ static inline void print_server_reply(struct connection *c) {
 
 	cmd = serverCARD8(1);
 	seq = serverCARD16(2);
-	for( lastp = &c->expectedreplies ; 
+	for( lastp = &c->expectedreplies ;
 			(replyto=*lastp) != NULL ; lastp=&replyto->next){
 		if( (replyto->seq & 0xFFFFFFFF ) == seq ) {
 			bool ignore = false, dontremove = false;
@@ -1186,10 +1185,10 @@ static inline void print_server_reply(struct connection *c) {
 				replyto->from->reply_func(c,&ignore,&dontremove,replyto->data);
 
 			if( !ignore ) {
-				printf("%03d:>:0x%04x:%u: Reply to %s: ", c->id, seq, (unsigned int)c->serverignore,replyto->from->name);
+				fprintf(out,"%03d:>:0x%04x:%u: Reply to %s: ", c->id, seq, (unsigned int)c->serverignore,replyto->from->name);
 				print_parameters(c,
 					c->serverbuffer,len,replyto->from->answers,false,&stack);
-				putchar('\n');
+				putc('\n',out);
 			}
 			if( !dontremove ) {
 				*lastp = replyto->next;
@@ -1201,7 +1200,7 @@ static inline void print_server_reply(struct connection *c) {
 			return;
 		}
 	}
-	printf("%03d:>:%04x:%u: unexpected reply\n",
+	fprintf(out,"%03d:>:%04x:%u: unexpected reply\n",
 			c->id, seq, c->serverignore);
 			
 }
@@ -1235,7 +1234,7 @@ static inline void print_server_error(struct connection *c) {
 		}
 
 	}
-	printf("%03d:>:%u:Error %hhu=%s: major=%u, minor=%u, bad=%u\n",
+	fprintf(out,"%03d:>:%u:Error %hhu=%s: major=%u, minor=%u, bad=%u\n",
 			c->id,
 			(int)serverCARD16(2),
 			cmd,
@@ -1271,7 +1270,7 @@ void parse_client(struct connection *c) {
 		 }
 		 c->clientignore =  l;
 
-		 printf("%03d:<: am %s want %d:%d authorising with '%*s' of length %d\n",
+		 fprintf(out,"%03d:<: am %s want %d:%d authorising with '%*s' of length %d\n",
 				 c->id,
 				 c->bigendian?"msb-first":"lsb-first",
 				 (int)clientCARD16(2),
@@ -1283,23 +1282,23 @@ void parse_client(struct connection *c) {
 		 return;
 	 case c_normal:
 		 if( c->clientcount < 4 ) {
-			 printf("%03d:<: Warning: Waiting for rest of package (yet only got %u)!\n",c->id,c->clientcount);
+			 fprintf(out,"%03d:<: Warning: Waiting for rest of package (yet only got %u)!\n",c->id,c->clientcount);
 			 return;
 		 }
 		 l = 4*clientCARD16(2);
 		 if( l == 0 ) {
 			 if( c->clientcount < 8 ) {
-				 printf("%03d:<: Warning: Waiting for rest of package (yet only got %u)!\n",c->id,c->clientcount);
+				 fprintf(out,"%03d:<: Warning: Waiting for rest of package (yet only got %u)!\n",c->id,c->clientcount);
 				 return;
 			 }
 			 l = 4*clientCARD32(4);
 			 bigrequest = true;
 		 } else
 			 bigrequest = false;
-		 if( c->clientcount == sizeof(c->clientbuffer) ) 
-			 printf("%03d:<: Warning: buffer filled!\n",c->id);
+		 if( c->clientcount == sizeof(c->clientbuffer) )
+			 fprintf(out,"%03d:<: Warning: buffer filled!\n",c->id);
 		 else if( c->clientcount < l ) {
-			 printf("%03d:<: Warning: Waiting for rest of package (yet got %u of %u)!\n",c->id,c->clientcount,(unsigned int)l);
+			 fprintf(out,"%03d:<: Warning: Waiting for rest of package (yet got %u of %u)!\n",c->id,c->clientcount,(unsigned int)l);
 			 return;
 		 }
 		 c->clientignore = l;
@@ -1332,19 +1331,19 @@ void parse_server(struct connection *c) {
 		 cmd = serverCARD16(0);
 		 switch( cmd ) {
 		  case 0:
-			  printf("%03d:>: Failed, version is %d:%d reason is '%*s'.\n",
+			  fprintf(out,"%03d:>: Failed, version is %d:%d reason is '%*s'.\n",
 					 c->id,
 					 (int)serverCARD16(2),
 					 (int)serverCARD16(4),
 					 (int)(4*len),
 					 &c->serverbuffer[8]);
 		  case 2:
-			  printf("%03d:>: More authentication needed, reason is '%*s'.\n",
+			  fprintf(out,"%03d:>: More authentication needed, reason is '%*s'.\n",
 					 c->id,
 					 (int)(4*len),
 					 &c->serverbuffer[8]);
 		  case 1:
-			  printf("%03d:>: Success, version is %d:%d-%d.\n",
+			  fprintf(out,"%03d:>: Success, version is %d:%d-%d.\n",
 					 c->id,
 					 (int)serverCARD16(2),
 					 (int)serverCARD16(4),
@@ -1379,34 +1378,34 @@ void parse_server(struct connection *c) {
 
 static void print_event(struct connection *c,const unsigned char *buffer) {
 	const struct event *event;
-	u_int8_t code = getCARD8(0);
+	uint8_t code = getCARD8(0);
 	unsigned long stackvalues[30];
 	struct stack stack;
 	stack.base = stackvalues;
 	stack.num = 30;
 	stack.ofs = 0;
 
-	if( (code & 0x80) != 0 ) 
-		fputs("(generated) ",stdout);
+	if( (code & 0x80) != 0 )
+		fputs("(generated) ",out);
 	code &= 0x7F;
 	if( code <= 1 || code > NUM(events) ) {
 		struct usedextension *u = c->usedextensions;
 		while( u != NULL ) {
-			if( code >= u->first_event && 
+			if( code >= u->first_event &&
 			    code-u->first_event < u->extension->numevents) {
-				event = u->extension->events + 
+				event = u->extension->events +
 						(code-u->first_event);
 				break;
 			}
 			u = u->next;
 		}
 		if( u == NULL ) {
-			printf("unknown code %hhu",code);
+			fprintf(out,"unknown code %hhu",code);
 			return;
 		}
 	} else
 		event = &events[code];
-	printf("%s(%hhu) ",event->name,code);
+	fprintf(out,"%s(%hhu) ",event->name,code);
 	print_parameters(c,buffer,32,event->parameters,false,&stack);
 }
 
@@ -1415,6 +1414,7 @@ static void print_event(struct connection *c,const unsigned char *buffer) {
 #include "render.inc"
 #include "randr.inc"
 #include "mitshm.inc"
+#include "xf86vidmode.inc"
 
 #define EXT(a,b) { a , sizeof(a)-1, \
 	extension ## b, NUM(extension ## b), \
@@ -1425,7 +1425,8 @@ struct extension extensions[] = {
 	EXT("RANDR",RANDR),
 	EXT("RENDER",RENDER),
 	EXT("SHAPE",SHAPE),
-	EXT("BIG-REQUESTS",BIGREQUEST)
+	EXT("BIG-REQUESTS",BIGREQUEST),
+	EXT("XFree86-VidModeExtension",XF86VidMode)
 };
 #undef EXT
 
